@@ -720,11 +720,6 @@ int main(int argc, char **argv) {
     aws_host_resolver_release(resolver);
     aws_event_loop_group_release(el_group);
 
-    if (aws_global_thread_creator_shutdown_wait_for(5)) {
-        fprintf(stderr, "Timeout waiting for thread shutdown!");
-        exit(1);
-    }
-
     if (tls_ctx) {
         aws_tls_connection_options_clean_up(&tls_connection_options);
         aws_tls_ctx_release(tls_ctx);
