@@ -4,6 +4,8 @@
  */
 #include <aws/io/tls_channel_handler.h>
 
+#include <aws/common/clock.h>
+
 #include <aws/io/channel.h>
 #include <aws/io/event_loop.h>
 #include <aws/io/file_utils.h>
@@ -32,6 +34,12 @@
 static const char *s_default_ca_dir = NULL;
 static const char *s_default_ca_file = NULL;
 
+struct s2n_delayed_shutdown_task {
+    struct aws_channel_task task;
+    struct aws_channel_slot *slot;
+    int error;
+};
+
 struct s2n_handler {
     struct aws_channel_handler handler;
     struct aws_tls_channel_handler_shared shared_state;
@@ -49,6 +57,7 @@ struct s2n_handler {
     void *user_data;
     bool advertise_alpn_message;
     bool negotiation_finished;
+    struct s2n_delayed_shutdown_task delayed_shutdown_task;
 };
 
 struct s2n_ctx {
@@ -207,20 +216,19 @@ static int s_generic_send(struct s2n_handler *handler, struct aws_byte_buf *buf)
 
     size_t processed = 0;
     while (processed < buf->len) {
+        const size_t overhead = aws_channel_slot_upstream_message_overhead(handler->slot);
+        const size_t message_size_hint = (buf->len - processed) + overhead;
         struct aws_io_message *message = aws_channel_acquire_message_from_pool(
-            handler->slot->channel, AWS_IO_MESSAGE_APPLICATION_DATA, buf->len - processed);
+            handler->slot->channel, AWS_IO_MESSAGE_APPLICATION_DATA, message_size_hint);
 
-        if (!message) {
+        if (!message || message->message_data.capacity <= overhead) {
             errno = ENOMEM;
             return -1;
         }
 
-        const size_t overhead = aws_channel_slot_upstream_message_overhead(handler->slot);
-        const size_t available_msg_write_capacity = buffer_cursor.len - overhead;
-
-        const size_t to_write = message->message_data.capacity > available_msg_write_capacity
-                                    ? available_msg_write_capacity
-                                    : message->message_data.capacity;
+        const size_t available_msg_write_capacity = message->message_data.capacity - overhead;
+        const size_t to_write =
+            available_msg_write_capacity >= buffer_cursor.len ? buffer_cursor.len : available_msg_write_capacity;
 
         struct aws_byte_cursor chunk = aws_byte_cursor_advance(&buffer_cursor, to_write);
         if (aws_byte_buf_append(&message->message_data, &chunk)) {
@@ -510,6 +518,47 @@ static int s_s2n_handler_process_write_message(
     return AWS_OP_SUCCESS;
 }
 
+static void s_delayed_shutdown_task_fn(struct aws_channel_task *channel_task, void *arg, enum aws_task_status status) {
+    (void)channel_task;
+
+    struct aws_channel_handler *handler = arg;
+    struct s2n_handler *s2n_handler = handler->impl;
+
+    if (status == AWS_TASK_STATUS_RUN_READY) {
+        AWS_LOGF_DEBUG(AWS_LS_IO_TLS, "id=%p: Delayed shut down in write direction", (void *)handler)
+        s2n_blocked_status blocked;
+        /* make a best effort, but the channel is going away after this run, so.... you only get one shot anyways */
+        s2n_shutdown(s2n_handler->connection, &blocked);
+    }
+    aws_channel_slot_on_handler_shutdown_complete(
+        s2n_handler->delayed_shutdown_task.slot,
+        AWS_CHANNEL_DIR_WRITE,
+        s2n_handler->delayed_shutdown_task.error,
+        false);
+}
+
+static int s_s2n_do_delayed_shutdown(
+    struct aws_channel_handler *handler,
+    struct aws_channel_slot *slot,
+    int error_code) {
+    struct s2n_handler *s2n_handler = (struct s2n_handler *)handler->impl;
+
+    s2n_handler->delayed_shutdown_task.slot = slot;
+    s2n_handler->delayed_shutdown_task.error = error_code;
+
+    uint64_t shutdown_delay = s2n_connection_get_delay(s2n_handler->connection);
+    uint64_t now = 0;
+
+    if (aws_channel_current_clock_time(slot->channel, &now)) {
+        return AWS_OP_ERR;
+    }
+
+    uint64_t shutdown_time = aws_add_u64_saturating(shutdown_delay, now);
+    aws_channel_schedule_task_future(slot->channel, &s2n_handler->delayed_shutdown_task.task, shutdown_time);
+
+    return AWS_OP_SUCCESS;
+}
+
 static int s_s2n_handler_shutdown(
     struct aws_channel_handler *handler,
     struct aws_channel_slot *slot,
@@ -520,10 +569,10 @@ static int s_s2n_handler_shutdown(
 
     if (dir == AWS_CHANNEL_DIR_WRITE) {
         if (!abort_immediately && error_code != AWS_IO_SOCKET_CLOSED) {
-            AWS_LOGF_DEBUG(AWS_LS_IO_TLS, "id=%p: Shutting down write direction", (void *)handler)
-            s2n_blocked_status blocked;
-            /* make a best effort, but the channel is going away after this run, so.... you only get one shot anyways */
-            s2n_shutdown(s2n_handler->connection, &blocked);
+            AWS_LOGF_DEBUG(AWS_LS_IO_TLS, "id=%p: Scheduling delayed write direction shutdown", (void *)handler)
+            if (s_s2n_do_delayed_shutdown(handler, slot, error_code) == AWS_OP_SUCCESS) {
+                return AWS_OP_SUCCESS;
+            }
         }
     } else {
         AWS_LOGF_DEBUG(
@@ -684,9 +733,11 @@ static size_t s_tl_cleanup_key = 0; /* Address of variable serves as key in hash
  * This local object is added to the table of every event loop that has a (s2n) tls connection
  * added to it at some point in time
  */
-static struct aws_event_loop_local_object s_tl_cleanup_object = {.key = &s_tl_cleanup_key,
-                                                                 .object = NULL,
-                                                                 .on_object_removed = NULL};
+static struct aws_event_loop_local_object s_tl_cleanup_object = {
+    .key = &s_tl_cleanup_key,
+    .object = NULL,
+    .on_object_removed = NULL,
+};
 
 static void s_aws_cleanup_s2n_thread_local_state(void *user_data) {
     (void)user_data;
@@ -809,6 +860,12 @@ static struct aws_channel_handler *s_new_tls_handler(
         goto cleanup_conn;
     }
 
+    aws_channel_task_init(
+        &s2n_handler->delayed_shutdown_task.task,
+        s_delayed_shutdown_task_fn,
+        &s2n_handler->handler,
+        "s2n_delayed_shutdown");
+
     if (s_s2n_tls_channel_handler_schedule_thread_local_cleanup(slot)) {
         goto cleanup_conn;
     }
@@ -847,6 +904,26 @@ static void s_s2n_ctx_destroy(struct s2n_ctx *s2n_ctx) {
     }
 }
 
+static int s2n_wall_clock_time_nanoseconds(void *context, uint64_t *time_in_ns) {
+    (void)context;
+    if (aws_sys_clock_get_ticks(time_in_ns)) {
+        *time_in_ns = 0;
+        return -1;
+    }
+
+    return 0;
+}
+
+static int s2n_monotonic_clock_time_nanoseconds(void *context, uint64_t *time_in_ns) {
+    (void)context;
+    if (aws_high_res_clock_get_ticks(time_in_ns)) {
+        *time_in_ns = 0;
+        return -1;
+    }
+
+    return 0;
+}
+
 static struct aws_tls_ctx *s_tls_ctx_new(
     struct aws_allocator *alloc,
     const struct aws_tls_ctx_options *options,
@@ -872,6 +949,16 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         goto cleanup_s2n_ctx;
     }
 
+    int set_clock_result = s2n_config_set_wall_clock(s2n_ctx->s2n_config, s2n_wall_clock_time_nanoseconds, NULL);
+    if (set_clock_result != S2N_ERR_T_OK) {
+        goto cleanup_s2n_config;
+    }
+
+    set_clock_result = s2n_config_set_monotonic_clock(s2n_ctx->s2n_config, s2n_monotonic_clock_time_nanoseconds, NULL);
+    if (set_clock_result != S2N_ERR_T_OK) {
+        goto cleanup_s2n_config;
+    }
+
     switch (options->minimum_tls_version) {
         case AWS_IO_SSLv3:
             s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "CloudFront-SSL-v-3");
@@ -889,7 +976,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
             AWS_LOGF_ERROR(AWS_LS_IO_TLS, "TLS 1.3 is not supported yet.");
             /* sorry guys, we'll add this as soon as s2n does. */
             aws_raise_error(AWS_IO_TLS_VERSION_UNSUPPORTED);
-            goto cleanup_s2n_ctx;
+            goto cleanup_s2n_config;
         case AWS_IO_TLS_VER_SYS_DEFAULTS:
         default:
             s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "ELBSecurityPolicy-TLS-1-1-2017-01");
@@ -917,7 +1004,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         default:
             AWS_LOGF_ERROR(AWS_LS_IO_TLS, "Unrecognized TLS Cipher Preference: %d", options->cipher_pref);
             aws_raise_error(AWS_IO_TLS_CIPHER_PREF_UNSUPPORTED);
-            goto cleanup_s2n_ctx;
+            goto cleanup_s2n_config;
     }
 
     if (options->certificate.len && options->private_key.len) {
@@ -926,13 +1013,13 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         if (!aws_text_is_utf8(options->certificate.buffer, options->certificate.len)) {
             AWS_LOGF_ERROR(AWS_LS_IO_TLS, "static: failed to import certificate, must be ASCII/UTF-8 encoded");
             aws_raise_error(AWS_IO_FILE_VALIDATION_FAILURE);
-            goto cleanup_s2n_ctx;
+            goto cleanup_s2n_config;
         }
 
         if (!aws_text_is_utf8(options->private_key.buffer, options->private_key.len)) {
             AWS_LOGF_ERROR(AWS_LS_IO_TLS, "static: failed to import private key, must be ASCII/UTF-8 encoded");
             aws_raise_error(AWS_IO_FILE_VALIDATION_FAILURE);
-            goto cleanup_s2n_ctx;
+            goto cleanup_s2n_config;
         }
 
         int err_code = s2n_config_add_cert_chain_and_key(
