@@ -608,8 +608,6 @@ static int s_test_connect_timeout(struct aws_allocator *allocator, void *ctx) {
     aws_socket_clean_up(&outgoing);
     aws_event_loop_group_release(el_group);
 
-    ASSERT_SUCCESS(aws_global_thread_creator_shutdown_wait_for(10));
-
     aws_io_library_clean_up();
 
     return 0;
@@ -683,7 +681,8 @@ static int s_test_connect_timeout_cancelation(struct aws_allocator *allocator, v
     ASSERT_SUCCESS(aws_socket_connect(&outgoing, &endpoint, event_loop, s_local_outgoing_connection, &outgoing_args));
 
     aws_event_loop_group_release(el_group);
-    ASSERT_SUCCESS(aws_global_thread_creator_shutdown_wait_for(10));
+
+    aws_thread_join_all_managed();
 
     ASSERT_INT_EQUALS(AWS_IO_EVENT_LOOP_SHUTDOWN, outgoing_args.last_error);
     aws_socket_clean_up(&outgoing);
@@ -1052,7 +1051,6 @@ static int s_cleanup_before_connect_or_timeout_doesnt_explode(struct aws_allocat
     ASSERT_FALSE(outgoing_args.error_invoked);
 
     aws_event_loop_group_release(el_group);
-    ASSERT_SUCCESS(aws_global_thread_creator_shutdown_wait_for(10));
 
     aws_io_library_clean_up();
 
@@ -1326,6 +1324,232 @@ static int s_cleanup_in_write_cb_doesnt_explode(struct aws_allocator *allocator,
 }
 AWS_TEST_CASE(cleanup_in_write_cb_doesnt_explode, s_cleanup_in_write_cb_doesnt_explode)
 
+/* stuff for the sock_write_cb_is_async test */
+enum async_role {
+    ASYNC_ROLE_A_CALLBACK_WRITES_D,
+    ASYNC_ROLE_B_CALLBACK_CLEANS_UP_SOCKET,
+    ASYNC_ROLE_C_IS_LAST_FROM_INITIAL_BATCH_OF_WRITES,
+    ASYNC_ROLE_D_GOT_WRITTEN_VIA_CALLBACK,
+    ASYNC_ROLE_COUNT
+};
+
+static struct {
+    struct aws_allocator *allocator;
+    struct aws_event_loop *event_loop;
+    struct aws_socket *write_socket;
+    struct aws_socket *read_socket;
+    bool currently_writing;
+    enum async_role next_expected_callback;
+
+    struct aws_mutex *mutex;
+    struct aws_condition_variable *condition_variable;
+    bool write_tasks_complete;
+    bool read_tasks_complete;
+} g_async_tester;
+
+static bool s_async_tasks_complete_pred(void *arg) {
+    (void)arg;
+    return g_async_tester.write_tasks_complete && g_async_tester.read_tasks_complete;
+}
+
+/* read until socket gets hung up on */
+static void s_async_read_task(struct aws_task *task, void *args, enum aws_task_status status) {
+    (void)args;
+    (void)status;
+    uint8_t buf_storage[100];
+    struct aws_byte_buf buf = aws_byte_buf_from_array(buf_storage, sizeof(buf_storage));
+    while (true) {
+        size_t amount_read = 0;
+        buf.len = 0;
+        if (aws_socket_read(g_async_tester.read_socket, &buf, &amount_read)) {
+            /* reschedule task to try reading more later */
+            if (AWS_IO_READ_WOULD_BLOCK == aws_last_error()) {
+                aws_event_loop_schedule_task_now(g_async_tester.event_loop, task);
+                break;
+            }
+
+            /* other end must have hung up. clean up and signal completion */
+            aws_socket_clean_up(g_async_tester.read_socket);
+            aws_mem_release(g_async_tester.allocator, g_async_tester.read_socket);
+
+            aws_mutex_lock(g_async_tester.mutex);
+            g_async_tester.read_tasks_complete = true;
+            aws_mutex_unlock(g_async_tester.mutex);
+            aws_condition_variable_notify_all(g_async_tester.condition_variable);
+            break;
+        }
+    }
+}
+
+static void s_async_write_completion(struct aws_socket *socket, int error_code, size_t bytes_written, void *user_data) {
+    enum async_role role = *(enum async_role *)user_data;
+    aws_mem_release(g_async_tester.allocator, user_data);
+
+    /* ensure callback is not firing synchronously from within aws_socket_write() */
+    AWS_FATAL_ASSERT(!g_async_tester.currently_writing);
+
+    /* ensure callbacks arrive in order */
+    AWS_FATAL_ASSERT(g_async_tester.next_expected_callback == role);
+    g_async_tester.next_expected_callback++;
+
+    switch (role) {
+        case ASYNC_ROLE_A_CALLBACK_WRITES_D: {
+            AWS_FATAL_ASSERT(0 == error_code);
+            AWS_FATAL_ASSERT(1 == bytes_written);
+            g_async_tester.currently_writing = true;
+            struct aws_byte_cursor data = aws_byte_cursor_from_c_str("D");
+            enum async_role *d_role = aws_mem_acquire(g_async_tester.allocator, sizeof(enum async_role));
+            *d_role = ASYNC_ROLE_D_GOT_WRITTEN_VIA_CALLBACK;
+            AWS_FATAL_ASSERT(0 == aws_socket_write(socket, &data, s_async_write_completion, d_role));
+            g_async_tester.currently_writing = false;
+            break;
+        }
+        case ASYNC_ROLE_B_CALLBACK_CLEANS_UP_SOCKET:
+            AWS_FATAL_ASSERT(0 == error_code);
+            AWS_FATAL_ASSERT(1 == bytes_written);
+            aws_socket_clean_up(socket);
+            break;
+        case ASYNC_ROLE_C_IS_LAST_FROM_INITIAL_BATCH_OF_WRITES:
+            /* C might succeed or fail (since socket killed after B completes), either is valid */
+            break;
+        case ASYNC_ROLE_D_GOT_WRITTEN_VIA_CALLBACK:
+            /* write tasks complete! */
+            aws_mutex_lock(g_async_tester.mutex);
+            g_async_tester.write_tasks_complete = true;
+            aws_mutex_unlock(g_async_tester.mutex);
+            aws_condition_variable_notify_all(g_async_tester.condition_variable);
+            break;
+        default:
+            AWS_FATAL_ASSERT(0);
+    }
+}
+
+static void s_async_write_task(struct aws_task *task, void *args, enum aws_task_status status) {
+    (void)task;
+    (void)args;
+    (void)status;
+
+    g_async_tester.currently_writing = true;
+
+    struct aws_byte_cursor data = aws_byte_cursor_from_c_str("A");
+    enum async_role *role = aws_mem_acquire(g_async_tester.allocator, sizeof(enum async_role));
+    *role = ASYNC_ROLE_A_CALLBACK_WRITES_D;
+    AWS_FATAL_ASSERT(0 == aws_socket_write(g_async_tester.write_socket, &data, s_async_write_completion, role));
+
+    data = aws_byte_cursor_from_c_str("B");
+    role = aws_mem_acquire(g_async_tester.allocator, sizeof(enum async_role));
+    *role = ASYNC_ROLE_B_CALLBACK_CLEANS_UP_SOCKET;
+    AWS_FATAL_ASSERT(0 == aws_socket_write(g_async_tester.write_socket, &data, s_async_write_completion, role));
+
+    data = aws_byte_cursor_from_c_str("C");
+    role = aws_mem_acquire(g_async_tester.allocator, sizeof(enum async_role));
+    *role = ASYNC_ROLE_C_IS_LAST_FROM_INITIAL_BATCH_OF_WRITES;
+    AWS_FATAL_ASSERT(0 == aws_socket_write(g_async_tester.write_socket, &data, s_async_write_completion, role));
+
+    g_async_tester.currently_writing = false;
+}
+
+/**
+ * aws_socket_write()'s completion callback MUST fire asynchronously.
+ * Otherwise, we can get multiple write() calls in the same callstack, which
+ * leads to esoteric bugs (https://github.com/aws/aws-iot-device-sdk-cpp-v2/issues/194).
+ */
+static int s_sock_write_cb_is_async(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    /* set up server (read) and client (write) sockets */
+    struct aws_event_loop *event_loop = aws_event_loop_new_default(allocator, aws_high_res_clock_get_ticks);
+
+    ASSERT_NOT_NULL(event_loop, "Event loop creation failed with error: %s", aws_error_debug_str(aws_last_error()));
+    ASSERT_SUCCESS(aws_event_loop_run(event_loop));
+
+    struct aws_mutex mutex = AWS_MUTEX_INIT;
+    struct aws_condition_variable condition_variable = AWS_CONDITION_VARIABLE_INIT;
+
+    struct local_listener_args listener_args = {
+        .mutex = &mutex,
+        .condition_variable = &condition_variable,
+        .incoming = NULL,
+        .incoming_invoked = false,
+        .error_invoked = false,
+    };
+
+    struct aws_socket_options options;
+    AWS_ZERO_STRUCT(options);
+    options.connect_timeout_ms = 3000;
+    options.keepalive = true;
+    options.keep_alive_interval_sec = 1000;
+    options.keep_alive_timeout_sec = 60000;
+    options.type = AWS_SOCKET_STREAM;
+    options.domain = AWS_SOCKET_LOCAL;
+
+    uint64_t timestamp = 0;
+    ASSERT_SUCCESS(aws_sys_clock_get_ticks(&timestamp));
+    struct aws_socket_endpoint endpoint;
+    AWS_ZERO_STRUCT(endpoint);
+    snprintf(endpoint.address, sizeof(endpoint.address), LOCAL_SOCK_TEST_PATTERN, (long long unsigned)timestamp);
+
+    struct aws_socket listener;
+    ASSERT_SUCCESS(aws_socket_init(&listener, allocator, &options));
+
+    ASSERT_SUCCESS(aws_socket_bind(&listener, &endpoint));
+    ASSERT_SUCCESS(aws_socket_listen(&listener, 1024));
+    ASSERT_SUCCESS(aws_socket_start_accept(&listener, event_loop, s_local_listener_incoming, &listener_args));
+
+    struct local_outgoing_args outgoing_args = {
+        .mutex = &mutex, .condition_variable = &condition_variable, .connect_invoked = false, .error_invoked = false};
+
+    struct aws_socket outgoing;
+    ASSERT_SUCCESS(aws_socket_init(&outgoing, allocator, &options));
+    ASSERT_SUCCESS(aws_socket_connect(&outgoing, &endpoint, event_loop, s_local_outgoing_connection, &outgoing_args));
+
+    ASSERT_SUCCESS(aws_mutex_lock(&mutex));
+    ASSERT_SUCCESS(aws_condition_variable_wait_pred(&condition_variable, &mutex, s_incoming_predicate, &listener_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
+        &condition_variable, &mutex, s_connection_completed_predicate, &outgoing_args));
+    ASSERT_SUCCESS(aws_mutex_unlock(&mutex));
+
+    ASSERT_TRUE(listener_args.incoming_invoked);
+    ASSERT_FALSE(listener_args.error_invoked);
+    struct aws_socket *server_sock = listener_args.incoming;
+    ASSERT_TRUE(outgoing_args.connect_invoked);
+    ASSERT_FALSE(outgoing_args.error_invoked);
+    ASSERT_INT_EQUALS(options.domain, listener_args.incoming->options.domain);
+    ASSERT_INT_EQUALS(options.type, listener_args.incoming->options.type);
+
+    ASSERT_SUCCESS(aws_socket_assign_to_event_loop(server_sock, event_loop));
+    aws_socket_subscribe_to_readable_events(server_sock, s_on_readable, NULL);
+    aws_socket_subscribe_to_readable_events(&outgoing, s_on_readable, NULL);
+
+    /* set up g_async_tester */
+    g_async_tester.allocator = allocator;
+    g_async_tester.event_loop = event_loop;
+    g_async_tester.write_socket = &outgoing;
+    g_async_tester.read_socket = server_sock;
+    g_async_tester.mutex = &mutex;
+    g_async_tester.condition_variable = &condition_variable;
+
+    /* kick off writer and reader tasks */
+    struct aws_task writer_task;
+    aws_task_init(&writer_task, s_async_write_task, NULL, "async_test_write_task");
+    aws_event_loop_schedule_task_now(event_loop, &writer_task);
+
+    struct aws_task reader_task;
+    aws_task_init(&reader_task, s_async_read_task, NULL, "async_test_read_task");
+    aws_event_loop_schedule_task_now(event_loop, &reader_task);
+
+    /* wait for tasks to complete */
+    aws_mutex_lock(&mutex);
+    aws_condition_variable_wait_pred(&condition_variable, &mutex, s_async_tasks_complete_pred, NULL);
+    aws_mutex_unlock(&mutex);
+
+    /* cleanup */
+    aws_socket_clean_up(&listener);
+    aws_event_loop_destroy(event_loop);
+    return 0;
+}
+AWS_TEST_CASE(sock_write_cb_is_async, s_sock_write_cb_is_async)
+
 #ifdef _WIN32
 static int s_local_socket_pipe_connected_race(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
