@@ -23,15 +23,15 @@ static const size_t s_default_body_streaming_priority_queue_size = 16;
 static int s_s3_request_priority_queue_pred(const void *a, const void *b);
 static void s_s3_meta_request_destroy(void *user_data);
 
-static void s_s3_meta_request_send_request(struct aws_s3_client *client, struct aws_s3_vip_connection *vip_connection);
-
 static void s_s3_meta_request_init_signing_date_time(
     struct aws_s3_meta_request *meta_request,
     struct aws_date_time *date_time);
 
-static int s_s3_meta_request_sign_request(
+static void s_s3_meta_request_sign_request(
     struct aws_s3_meta_request *meta_request,
-    struct aws_s3_vip_connection *vip_connection);
+    struct aws_s3_request *request,
+    aws_signing_complete_fn *on_signing_complete,
+    void *user_data);
 
 static void s_s3_meta_request_request_on_signed(
     struct aws_signing_result *signing_result,
@@ -57,18 +57,6 @@ static void s_s3_meta_request_send_request_finish(
     struct aws_http_stream *stream,
     int error_code);
 
-static void s_s3_meta_request_delivered_requests(
-    struct aws_s3_meta_request *meta_request,
-    int error_code,
-    uint32_t num_failed,
-    uint32_t num_successful);
-
-static void s_s3_meta_request_streaming_body_callback(
-    int error_code,
-    uint32_t num_failed,
-    uint32_t num_successful,
-    void *user_data);
-
 void aws_s3_meta_request_lock_synced_data(struct aws_s3_meta_request *meta_request) {
     AWS_PRECONDITION(meta_request);
 
@@ -109,6 +97,7 @@ int aws_s3_meta_request_init_base(
     AWS_ASSERT(vtable->send_request_finish);
 
     meta_request->allocator = allocator;
+    meta_request->type = options->type;
 
     /* Set up reference count. */
     aws_ref_count_init(&meta_request->ref_count, meta_request, s_s3_meta_request_destroy);
@@ -142,6 +131,7 @@ int aws_s3_meta_request_init_base(
     if (client != NULL) {
         aws_s3_client_acquire(client);
         meta_request->client = client;
+        meta_request->io_event_loop = aws_event_loop_group_get_next_loop(client->body_streaming_elg);
     }
 
     meta_request->synced_data.next_streaming_part = 1;
@@ -278,11 +268,19 @@ static void s_s3_meta_request_destroy(void *user_data) {
     aws_priority_queue_clean_up(&meta_request->synced_data.pending_body_streaming_requests);
     aws_s3_meta_request_result_clean_up(meta_request, &meta_request->synced_data.finish_result);
 
+    AWS_LOGF_DEBUG(
+        AWS_LS_S3_META_REQUEST, "id=%p Calling virtual meta request destroy function.", (void *)meta_request);
+
     meta_request->vtable->destroy(meta_request);
+    meta_request = NULL;
+
+    AWS_LOGF_DEBUG(AWS_LS_S3_META_REQUEST, "id=%p Calling meta request shutdown callback.", (void *)meta_request);
 
     if (shutdown_callback != NULL) {
         shutdown_callback(meta_request_user_data);
     }
+
+    AWS_LOGF_DEBUG(AWS_LS_S3_META_REQUEST, "id=%p Meta request clean up finished.", (void *)meta_request);
 }
 
 static int s_s3_request_priority_queue_pred(const void *a, const void *b) {
@@ -328,39 +326,126 @@ bool aws_s3_meta_request_is_finished(struct aws_s3_meta_request *meta_request) {
     return is_finished;
 }
 
-int aws_s3_meta_request_make_request(
-    struct aws_s3_meta_request *meta_request,
-    struct aws_s3_client *client,
-    struct aws_s3_vip_connection *vip_connection) {
-    AWS_PRECONDITION(meta_request);
-    AWS_PRECONDITION(client);
-    AWS_PRECONDITION(vip_connection);
+static void s_s3_meta_request_prepare_request_task(struct aws_task *task, void *arg, enum aws_task_status task_status);
 
-    struct aws_s3_request *request = vip_connection->request;
+static void s_s3_prepare_request_payload_callback_and_destroy(
+    struct aws_s3_prepare_request_payload *payload,
+    int error_code) {
+    AWS_PRECONDITION(payload);
+    AWS_PRECONDITION(payload->request);
+
+    struct aws_s3_meta_request *meta_request = payload->request->meta_request;
+    AWS_PRECONDITION(meta_request);
+
+    struct aws_s3_client *client = meta_request->client;
+    AWS_PRECONDITION(client);
+
+    struct aws_allocator *sba_allocator = client->sba_allocator;
+    AWS_PRECONDITION(sba_allocator);
+
+    /* Acquire a reference to the client pointer just be to safe, so that we know the callback won't trigger a clean up
+     * of the client and cause the sba_allocator to go away. */
+    aws_s3_client_acquire(client);
+
+    if (payload->callback != NULL) {
+        payload->callback(meta_request, payload->request, error_code, payload->user_data);
+    }
+
+    aws_mem_release(sba_allocator, payload);
+    aws_s3_client_release(client);
+}
+
+static void s_s3_meta_request_schedule_prepare_request_default(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request,
+    aws_s3_meta_request_prepare_request_callback_fn *callback,
+    void *user_data);
+
+void aws_s3_meta_request_prepare_request(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request,
+    aws_s3_meta_request_prepare_request_callback_fn *callback,
+    void *user_data) {
+    AWS_PRECONDITION(meta_request);
+    AWS_PRECONDITION(meta_request->vtable);
+
+    if (meta_request->vtable->schedule_prepare_request) {
+        meta_request->vtable->schedule_prepare_request(meta_request, request, callback, user_data);
+    } else {
+        s_s3_meta_request_schedule_prepare_request_default(meta_request, request, callback, user_data);
+    }
+}
+
+static void s_s3_meta_request_schedule_prepare_request_default(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request,
+    aws_s3_meta_request_prepare_request_callback_fn *callback,
+    void *user_data) {
+    AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(request);
 
+    struct aws_s3_client *client = meta_request->client;
+    AWS_PRECONDITION(client);
+
+    struct aws_allocator *sba_allocator = client->sba_allocator;
+    AWS_PRECONDITION(sba_allocator);
+
+    struct aws_s3_prepare_request_payload *payload =
+        aws_mem_calloc(sba_allocator, 1, sizeof(struct aws_s3_prepare_request_payload));
+
+    payload->request = request;
+    payload->callback = callback;
+    payload->user_data = user_data;
+
+    aws_task_init(
+        &payload->task, s_s3_meta_request_prepare_request_task, payload, "s3_meta_request_prepare_request_task");
+    aws_event_loop_schedule_task_now(meta_request->io_event_loop, &payload->task);
+}
+
+static void s_s3_meta_request_prepare_request_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
+    (void)task;
+    (void)task_status;
+
+    struct aws_s3_prepare_request_payload *payload = arg;
+    AWS_PRECONDITION(payload);
+
+    struct aws_s3_request *request = payload->request;
+    AWS_PRECONDITION(request);
+
+    struct aws_s3_meta_request *meta_request = request->meta_request;
+    AWS_PRECONDITION(meta_request);
+
     struct aws_s3_meta_request_vtable *vtable = meta_request->vtable;
     AWS_PRECONDITION(vtable);
 
-    if (vtable->prepare_request(meta_request, client, vip_connection, !vip_connection->is_retry)) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_META_REQUEST, "id=%p Could not prepare request %p", (void *)meta_request, (void *)request);
+    /* Client owns this event loop group. A cancel should not be possible. */
+    AWS_ASSERT(task_status == AWS_TASK_STATUS_RUN_READY);
 
-        goto call_finished_callback;
+    int prepare_result = vtable->prepare_request(meta_request, request);
+
+    ++request->num_times_prepared;
+
+    if (prepare_result == AWS_OP_ERR) {
+        int error_code = aws_last_error_or_unknown();
+
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p Could not prepare request %p due to error %d (%s).",
+            (void *)meta_request,
+            (void *)request,
+            error_code,
+            aws_error_str(error_code));
+
+        aws_s3_meta_request_lock_synced_data(meta_request);
+        aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
+        aws_s3_meta_request_unlock_synced_data(meta_request);
+
+        s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
+        return;
     }
 
     /* Sign the newly created message. */
-    if (s_s3_meta_request_sign_request(meta_request, vip_connection)) {
-        goto call_finished_callback;
-    }
-
-    return AWS_OP_SUCCESS;
-
-call_finished_callback:
-
-    s_s3_meta_request_send_request_finish(vip_connection, NULL, aws_last_error_or_unknown());
-
-    return AWS_OP_ERR;
+    s_s3_meta_request_sign_request(meta_request, request, s_s3_meta_request_request_on_signed, payload);
 }
 
 static void s_s3_meta_request_init_signing_date_time(
@@ -383,30 +468,30 @@ void aws_s3_meta_request_init_signing_date_time_default(
     aws_date_time_init_now(date_time);
 }
 
-static int s_s3_meta_request_sign_request(
+static void s_s3_meta_request_sign_request(
     struct aws_s3_meta_request *meta_request,
-    struct aws_s3_vip_connection *vip_connection) {
+    struct aws_s3_request *request,
+    aws_signing_complete_fn *on_signing_complete,
+    void *user_data) {
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(meta_request->vtable);
     AWS_PRECONDITION(meta_request->vtable->send_request_finish);
 
-    return meta_request->vtable->sign_request(meta_request, vip_connection);
+    meta_request->vtable->sign_request(meta_request, request, on_signing_complete, user_data);
 }
 
 /* Handles signing a message for the caller. */
-int aws_s3_meta_request_sign_request_default(
+void aws_s3_meta_request_sign_request_default(
     struct aws_s3_meta_request *meta_request,
-    struct aws_s3_vip_connection *vip_connection) {
-    AWS_PRECONDITION(meta_request)
-
-    AWS_PRECONDITION(vip_connection);
-    AWS_PRECONDITION(vip_connection->owning_vip);
-
-    struct aws_s3_client *client = vip_connection->owning_vip->owning_client;
-    AWS_PRECONDITION(client);
-
-    struct aws_s3_request *request = vip_connection->request;
+    struct aws_s3_request *request,
+    aws_signing_complete_fn *on_signing_complete,
+    void *user_data) {
+    AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(request);
+    AWS_PRECONDITION(on_signing_complete);
+
+    struct aws_s3_client *client = meta_request->client;
+    AWS_ASSERT(client);
 
     struct aws_signing_config_aws signing_config;
 
@@ -421,13 +506,12 @@ int aws_s3_meta_request_sign_request_default(
             (void *)meta_request,
             (void *)request);
 
-        s_s3_meta_request_request_on_signed(NULL, AWS_ERROR_SUCCESS, vip_connection);
-        return AWS_OP_SUCCESS;
+        on_signing_complete(NULL, AWS_ERROR_SUCCESS, user_data);
+        return;
     }
 
     s_s3_meta_request_init_signing_date_time(meta_request, &signing_config.date);
 
-    int result = AWS_OP_ERR;
     request->send_data.signable = aws_signable_new_http_request(meta_request->allocator, request->send_data.message);
 
     AWS_LOGF_DEBUG(
@@ -445,26 +529,23 @@ int aws_s3_meta_request_sign_request_default(
             (void *)meta_request,
             (void *)request);
 
-        goto done;
+        on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+        return;
     }
 
     if (aws_sign_request_aws(
             meta_request->allocator,
             request->send_data.signable,
             (struct aws_signing_config_base *)&signing_config,
-            s_s3_meta_request_request_on_signed,
-            vip_connection)) {
+            on_signing_complete,
+            user_data)) {
 
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST, "id=%p: Could not sign request %p", (void *)meta_request, (void *)request);
 
-        goto done;
+        on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+        return;
     }
-
-    result = AWS_OP_SUCCESS;
-
-done:
-    return result;
 }
 
 /* Handle the signing result, getting an HTTP connection for the request if signing succeeded. */
@@ -473,51 +554,47 @@ static void s_s3_meta_request_request_on_signed(
     int error_code,
     void *user_data) {
 
-    struct aws_s3_vip_connection *vip_connection = user_data;
-    AWS_PRECONDITION(vip_connection);
+    struct aws_s3_prepare_request_payload *payload = user_data;
+    AWS_PRECONDITION(payload);
 
-    struct aws_s3_client *client = vip_connection->owning_vip->owning_client;
-    AWS_PRECONDITION(client);
-
-    struct aws_s3_request *request = vip_connection->request;
+    struct aws_s3_request *request = payload->request;
     AWS_PRECONDITION(request);
 
     struct aws_s3_meta_request *meta_request = request->meta_request;
     AWS_PRECONDITION(meta_request);
 
     if (error_code != AWS_ERROR_SUCCESS) {
-        goto error_finish;
+        goto finish;
     }
 
     if (signing_result != NULL &&
         aws_apply_signing_result_to_http_request(request->send_data.message, meta_request->allocator, signing_result)) {
-        goto error_finish;
+
+        error_code = aws_last_error_or_unknown();
+        goto finish;
     }
 
-    AWS_LOGF_DEBUG(
-        AWS_LS_S3_META_REQUEST, "id=%p Getting HTTP connection for request %p", (void *)meta_request, (void *)request);
+finish:
 
-    s_s3_meta_request_send_request(client, vip_connection);
+    if (error_code != AWS_ERROR_SUCCESS) {
+        aws_s3_meta_request_lock_synced_data(meta_request);
+        aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
+        aws_s3_meta_request_unlock_synced_data(meta_request);
+    }
 
-    return;
-
-error_finish:
-
-    s_s3_meta_request_send_request_finish(vip_connection, NULL, aws_last_error_or_unknown());
+    s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
 }
 
-static void s_s3_meta_request_send_request(struct aws_s3_client *client, struct aws_s3_vip_connection *vip_connection) {
-    AWS_PRECONDITION(client);
+void aws_s3_meta_request_send_request(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_vip_connection *vip_connection) {
+    AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(vip_connection);
     AWS_PRECONDITION(vip_connection->http_connection);
-    (void)client;
 
     struct aws_s3_request *request = vip_connection->request;
     AWS_PRECONDITION(request);
 
-    struct aws_s3_meta_request *meta_request = request->meta_request;
-    AWS_PRECONDITION(meta_request);
-
     /* Now that we have a signed request and a connection, go ahead and issue the request. */
     struct aws_http_make_request_options options;
     AWS_ZERO_STRUCT(options);
@@ -792,19 +869,6 @@ void aws_s3_meta_request_send_request_finish_default(
     aws_s3_client_notify_connection_finished(client, vip_connection, error_code, finish_code);
 }
 
-static void s_s3_meta_request_streaming_body_callback(
-    int error_code,
-    uint32_t num_failed,
-    uint32_t num_successful,
-    void *user_data) {
-    (void)error_code;
-
-    struct aws_s3_meta_request *meta_request = user_data;
-    AWS_PRECONDITION(meta_request);
-
-    s_s3_meta_request_delivered_requests(meta_request, error_code, num_failed, num_successful);
-}
-
 void aws_s3_meta_request_finished_request(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request,
@@ -816,6 +880,26 @@ void aws_s3_meta_request_finished_request(
     meta_request->vtable->finished_request(meta_request, request, error_code);
 }
 
+struct s3_stream_response_body_payload {
+    struct aws_s3_meta_request *meta_request;
+    struct aws_linked_list requests;
+    struct aws_task task;
+};
+
+/* Pushes a request into the body streaming priority queue. Derived meta request types should not call this--they should
+ * instead call aws_s3_meta_request_stream_response_body_synced.*/
+static void s_s3_meta_request_body_streaming_push_synced(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request);
+
+/* Pops the next available request from the body streaming priority queue. If the parts previous the next request in the
+ * priority queue have not been placed in the priority queue yet, the priority queue will remain the same, and NULL will
+ * be returned. (Should not be needed to be called by derived types.) */
+static struct aws_s3_request *s_s3_meta_request_body_streaming_pop_next_synced(
+    struct aws_s3_meta_request *meta_request);
+
+static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *arg, enum aws_task_status task_status);
+
 void aws_s3_meta_request_stream_response_body_synced(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request) {
@@ -827,62 +911,104 @@ void aws_s3_meta_request_stream_response_body_synced(
     struct aws_linked_list streaming_requests;
     aws_linked_list_init(&streaming_requests);
 
-    if (aws_s3_meta_request_has_finish_result_synced(meta_request)) {
-        return;
-    }
+    /* Push it into the priority queue. */
+    s_s3_meta_request_body_streaming_push_synced(meta_request, request);
 
     struct aws_s3_client *client = meta_request->client;
-    AWS_ASSERT(client != NULL);
-
-    /* Push it into the priority queue. */
-    aws_s3_meta_request_body_streaming_push_synced(meta_request, request);
-
-    aws_atomic_fetch_add(&client->stats.num_requests_queued_waiting, 1);
+    AWS_PRECONDITION(client);
+    aws_atomic_fetch_add(&client->stats.num_requests_stream_queued_waiting, 1);
 
     /* Grab the next request that can be streamed back to the caller. */
-    struct aws_s3_request *next_streaming_request = aws_s3_meta_request_body_streaming_pop_synced(meta_request);
+    struct aws_s3_request *next_streaming_request = s_s3_meta_request_body_streaming_pop_next_synced(meta_request);
     uint32_t num_streaming_requests = 0;
 
     /* Grab any additional requests that could be streamed to the caller. */
     while (next_streaming_request != NULL) {
-        aws_atomic_fetch_sub(&client->stats.num_requests_queued_waiting, 1);
+        aws_atomic_fetch_sub(&client->stats.num_requests_stream_queued_waiting, 1);
 
         aws_linked_list_push_back(&streaming_requests, &next_streaming_request->node);
         ++num_streaming_requests;
-        next_streaming_request = aws_s3_meta_request_body_streaming_pop_synced(meta_request);
+        next_streaming_request = s_s3_meta_request_body_streaming_pop_next_synced(meta_request);
     }
 
-    if (!aws_linked_list_empty(&streaming_requests)) {
-        aws_s3_client_stream_response_body(
-            client, meta_request, &streaming_requests, s_s3_meta_request_streaming_body_callback, meta_request);
-
-        meta_request->synced_data.num_parts_delivery_sent += num_streaming_requests;
+    if (aws_linked_list_empty(&streaming_requests)) {
+        return;
     }
+
+    aws_atomic_fetch_add(&client->stats.num_requests_streaming, num_streaming_requests);
+
+    meta_request->synced_data.num_parts_delivery_sent += num_streaming_requests;
+
+    struct s3_stream_response_body_payload *payload =
+        aws_mem_calloc(client->sba_allocator, 1, sizeof(struct s3_stream_response_body_payload));
+
+    aws_s3_meta_request_acquire(meta_request);
+    payload->meta_request = meta_request;
+
+    aws_linked_list_init(&payload->requests);
+    aws_linked_list_swap_contents(&payload->requests, &streaming_requests);
+
+    aws_task_init(
+        &payload->task, s_s3_meta_request_body_streaming_task, payload, "s_s3_meta_request_body_streaming_task");
+    aws_event_loop_schedule_task_now(meta_request->io_event_loop, &payload->task);
 }
 
-static void s_s3_meta_request_delivered_requests(
-    struct aws_s3_meta_request *meta_request,
-    int error_code,
-    uint32_t num_failed,
-    uint32_t num_successful) {
+static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
+    (void)task;
+    (void)task_status;
+
+    struct s3_stream_response_body_payload *payload = arg;
+    AWS_PRECONDITION(payload);
+
+    struct aws_s3_meta_request *meta_request = payload->meta_request;
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(meta_request->vtable);
-    AWS_PRECONDITION(meta_request->vtable->delivered_requests);
 
-    meta_request->vtable->delivered_requests(meta_request, error_code, num_failed, num_successful);
-}
+    struct aws_s3_client *client = meta_request->client;
+    AWS_PRECONDITION(client);
 
-void aws_s3_meta_request_delivered_requests_default(
-    struct aws_s3_meta_request *meta_request,
-    int error_code,
-    uint32_t num_failed,
-    uint32_t num_successful) {
+    /* Client owns this event loop group. A cancel should not be possible. */
+    AWS_ASSERT(task_status == AWS_TASK_STATUS_RUN_READY);
 
-    AWS_PRECONDITION(meta_request);
-    AWS_PRECONDITION(meta_request->impl);
+    struct aws_linked_list completed_requests;
+    aws_linked_list_init(&completed_requests);
+
+    int error_code = AWS_ERROR_SUCCESS;
+    uint32_t num_successful = 0;
+    uint32_t num_failed = 0;
+
+    while (!aws_linked_list_empty(&payload->requests)) {
+        struct aws_linked_list_node *request_node = aws_linked_list_pop_front(&payload->requests);
+        struct aws_s3_request *request = AWS_CONTAINER_OF(request_node, struct aws_s3_request, node);
+        AWS_ASSERT(meta_request == request->meta_request);
+
+        struct aws_byte_cursor body_buffer_byte_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
+
+        AWS_ASSERT(request->part_number >= 1);
+
+        uint64_t range_start = (request->part_number - 1) * meta_request->part_size;
+
+        if (aws_s3_meta_request_has_finish_result(meta_request)) {
+            ++num_failed;
+        } else {
+            if (error_code == AWS_ERROR_SUCCESS && meta_request->body_callback &&
+                meta_request->body_callback(
+                    meta_request, &body_buffer_byte_cursor, range_start, meta_request->user_data)) {
+                error_code = aws_last_error_or_unknown();
+            }
+
+            if (error_code == AWS_ERROR_SUCCESS) {
+                ++num_successful;
+            } else {
+                ++num_failed;
+            }
+        }
+
+        aws_atomic_fetch_sub(&client->stats.num_requests_streaming, 1);
+        aws_s3_request_release(request);
+    }
 
     aws_s3_meta_request_lock_synced_data(meta_request);
-
     if (error_code != AWS_ERROR_SUCCESS) {
         aws_s3_meta_request_set_fail_synced(meta_request, NULL, error_code);
     }
@@ -891,9 +1017,15 @@ void aws_s3_meta_request_delivered_requests_default(
     meta_request->synced_data.num_parts_delivery_failed += num_failed;
     meta_request->synced_data.num_parts_delivery_succeeded += num_successful;
     aws_s3_meta_request_unlock_synced_data(meta_request);
+
+    aws_mem_release(client->sba_allocator, payload);
+    payload = NULL;
+
+    aws_s3_client_schedule_process_work(client);
+    aws_s3_meta_request_release(meta_request);
 }
 
-void aws_s3_meta_request_body_streaming_push_synced(
+static void s_s3_meta_request_body_streaming_push_synced(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request) {
     ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
@@ -907,7 +1039,8 @@ void aws_s3_meta_request_body_streaming_push_synced(
     aws_priority_queue_push(&meta_request->synced_data.pending_body_streaming_requests, &request);
 }
 
-struct aws_s3_request *aws_s3_meta_request_body_streaming_pop_synced(struct aws_s3_meta_request *meta_request) {
+static struct aws_s3_request *s_s3_meta_request_body_streaming_pop_next_synced(
+    struct aws_s3_meta_request *meta_request) {
     AWS_PRECONDITION(meta_request);
     ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
 
@@ -986,9 +1119,6 @@ unlock:
         aws_s3_request_release(release_request);
     }
 
-    aws_s3_client_release(meta_request->client);
-    meta_request->client = NULL;
-
     AWS_LOGF_DEBUG(
         AWS_LS_S3_META_REQUEST,
         "id=%p Meta request finished with error code %d (%s)",
@@ -996,16 +1126,15 @@ unlock:
         finish_result.error_code,
         aws_error_str(finish_result.error_code));
 
-    /* Grab a reference to the meta request before the finish callback to guarantee the meta request is around for
-     * the clean up call after.*/
-    aws_s3_meta_request_acquire(meta_request);
-
     if (meta_request->finish_callback != NULL) {
         meta_request->finish_callback(meta_request, &finish_result, meta_request->user_data);
     }
 
     aws_s3_meta_request_result_clean_up(meta_request, &finish_result);
-    aws_s3_meta_request_release(meta_request);
+
+    aws_s3_client_release(meta_request->client);
+    meta_request->client = NULL;
+    meta_request->io_event_loop = NULL;
 }
 
 int aws_s3_meta_request_read_body(struct aws_s3_meta_request *meta_request, struct aws_byte_buf *buffer) {
